﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MyNote001
{
    public partial class PictureAnalyzeAsCommand : Form
    {
        const System.Drawing.Imaging.PixelFormat FORMAT = System.Drawing.Imaging.PixelFormat.Format32bppArgb;

        enum AnalyzeType { Analyze1, Analyze2 };

        int previousA;
        int previousR;
        int previousG;
        int previousB;

        public PictureAnalyzeAsCommand()
        {
            InitializeComponent();
        }

        private Bitmap getBitmap_OpenFileDialog()
        {
            Bitmap bmp = null;

            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "画像ファイル(*.bmp;*.jpg;*.jpeg;*.png)|*.bmp;*.jpg;*.jpeg;*.png";
            ofd.Title = "開くファイルを選択してください";
            ofd.RestoreDirectory = true;

            if (ofd.ShowDialog() == DialogResult.OK)
            {
                System.IO.FileStream stream = new System.IO.FileStream(ofd.FileName, System.IO.FileMode.Open);
                bmp = new Bitmap(stream);
                stream.Close();
            }

            return bmp;
        }

        private void buttonLoadPicture_Click(object sender, EventArgs e)
        {
            Bitmap bmp = getBitmap_OpenFileDialog();
            if (bmp == null) return;

            int maxw = 300;
            int maxh = 300;
            if(bmp.Width > maxw || bmp.Height > maxh)
            {
                MessageBox.Show(string.Format("limit : {0} * {1}",maxw,maxh), 
                    string.Format("{0} * {1} is too large", bmp.Width, bmp.Height));
                return;
            }

            pictureBox1.Width = bmp.Width;
            pictureBox1.Height = bmp.Height;
            pictureBox1.Image = bmp;
            buttonAnalyze.Enabled = true;
            buttonRotate.Enabled = true;
            buttonCopyToClipboard.Enabled = true;
            richTextBox1.Text = "loaded";
        }

        private void buttonCopyToClipboard_Click(object sender, EventArgs e)
        {
            try
            {
                Clipboard.SetText(richTextBox1.Text);
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.ToString());
            }
        }

        private void buttonAnalyze_Click(object sender, EventArgs e)
        {
            if(pictureBox1.Image == null)
                return;

            Bitmap bmp = (Bitmap)pictureBox1.Image;

            richTextBox1.Text = "";
            StringBuilder sb = new StringBuilder();

            sbApendSub1(sb);
            sbApendSub2(sb, bmp, AnalyzeType.Analyze1);

            richTextBox1.Text = sb.ToString();
        }

        private void buttonRotate_Click(object sender, EventArgs e)
        {
            if (pictureBox1.Image == null)
                return;

            double rotate;
            try
            {
                rotate = double.Parse(textBox1.Text);
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.ToString());
                return;
            }

            Bitmap bmp = (Bitmap)pictureBox1.Image;
            int wh = (int)Math.Sqrt(bmp.Width * bmp.Width + bmp.Height * bmp.Height);

            pictureBox2.Width = wh;
            pictureBox2.Height = wh;
            Bitmap bmp_r = new Bitmap(wh, wh, FORMAT);

            double cos = Math.Cos( (-2) * Math.PI * rotate / 360.0);
            double sin = Math.Sin( (-2) * Math.PI * rotate / 360.0);

            for (int x = 0; x < wh; x++)
            {
                for (int y = 0; y < wh; y++)
                {
                    int ox = (int)(cos * (x - wh/2) - sin * (y - wh/2) + wh/2);
                    int oy = (int)(sin * (x - wh/2) + cos * (y - wh/2) + wh/2);
                    int ox2 = ox - (wh / 2 - bmp.Width / 2);
                    int oy2 = oy - (wh / 2 - bmp.Height / 2);
                    if (0 <= ox2 && ox2 < bmp.Width && 0 <= oy2 && oy2 < bmp.Height)
                    {
                        bmp_r.SetPixel(x,y,bmp.GetPixel(ox2,oy2));
                    }
                }
            }

            pictureBox2.Image = bmp_r;
            buttonAnalyze2.Enabled = true;

        }

        private void buttonAnalyze2_Click(object sender, EventArgs e)
        {
            if (pictureBox2.Image == null)
                return;

            Bitmap bmp = (Bitmap)pictureBox2.Image;

            richTextBox1.Text = "";
            StringBuilder sb = new StringBuilder();

            sbApendSub1(sb);
            sbApendSub2(sb, bmp, AnalyzeType.Analyze2);

            richTextBox1.Text = sb.ToString();
        }

        private void sbApendSub1(StringBuilder sb)
        {
            sb.Append("//,この二つは利用する立場からみた結果は同じ命令。");
            sb.Append(Environment.NewLine);
            sb.Append(MyMacros.DotDrawCycleReset);
            sb.Append(Environment.NewLine);
            sb.Append(MyMacros.DotDrawCycle + ",1");
            sb.Append(Environment.NewLine);

            sb.Append("//,ペンモード");
            sb.Append(Environment.NewLine);
            sb.Append(MyMacros.PenMode + ",s");
            sb.Append(Environment.NewLine);

            sb.Append("//,ドットドローモード");
            sb.Append(Environment.NewLine);
            sb.Append(MyMacros.DotDrawMode + ",ow");
            sb.Append(Environment.NewLine);

            sb.Append("//,アンチエイリアシング");
            sb.Append(Environment.NewLine);
            sb.Append(MyMacros.AntiAliasing + ",false");
            sb.Append(Environment.NewLine);

            sb.Append("//,ユーズストリングビルダー");
            sb.Append(Environment.NewLine);
            sb.Append(MacrosWatchAsCommand.USE_STRINGBUILDER);
            sb.Append(Environment.NewLine);

            sb.Append(Environment.NewLine);

            sb.Append("//,");
            sb.Append(Environment.NewLine);
        }

        private void sbApendSub2(StringBuilder sb, Bitmap bmp, AnalyzeType type)
        {
            previousA = previousR = previousG = previousB = -1;

            for (int y = 0; y < bmp.Height; y++)
            {
                for (int x = 0; x < bmp.Width; x++)
                {
                    Color c = bmp.GetPixel(x, y);

                    if (checkBoxA.Checked)
                    {
                        if (c.A == (byte)numericUpDownA.Value)
                        {
                            continue;
                        }
                    }

                    if (checkBoxRGB.Checked)
                    {
                        if (c.R == (byte)numericUpDownR.Value &&
                            c.G == (byte)numericUpDownG.Value &&
                            c.B == (byte)numericUpDownB.Value)
                        {
                            continue;
                        }
                    }

                    if (checkBoxAr.Checked)
                    {
                        byte min = (byte)Math.Min(numericUpDownAr1.Value, numericUpDownAr2.Value);
                        byte max = (byte)Math.Max(numericUpDownAr1.Value, numericUpDownAr2.Value);
                        if (min <= c.A && c.A <= max)
                        {
                            continue;
                        }
                    }

                    if (checkBoxZipColorCommand.Checked)
                    {
                        int nowA = c.A;
                        int nowR = c.R;
                        int nowG = c.G;
                        int nowB = c.B;
                        if (nowA == previousA && nowR == previousR && nowG == previousG && nowB == previousB)
                        {
                            ;
                        }
                        else
                        {
                            sb.Append(string.Format("{0},{1},{2},{3},{4}", MyMacros.Color, c.A, c.R, c.G, c.B));
                            sb.Append(Environment.NewLine);
                        }
                        previousA = nowA;
                        previousR = nowR;
                        previousG = nowG;
                        previousB = nowB;
                    }
                    else
                    {
                        sb.Append(string.Format("{0},{1},{2},{3},{4}", MyMacros.Color, c.A, c.R, c.G, c.B));
                        sb.Append(Environment.NewLine);
                    }

                    switch(type)
                    {
                        case AnalyzeType.Analyze1:
                    
                            sb.Append(string.Format("{0},{1},{2}", MyMacros.MouseUp, x, y));
                            break;
                        case AnalyzeType.Analyze2:
                            sb.Append(string.Format("{0},{1},{2}", MyMacros.MouseUp, x - bmp.Width / 2, y - bmp.Height / 2));
                            break;
                        default:
                            break;
                    }
                    sb.Append(Environment.NewLine);

                }
            }
        }
    }
}
