﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MyNote001
{
    public delegate void CpEventDelegate(object sender, System.EventArgs e);

    public partial class ColorPalette : Form
    {
        const System.Drawing.Imaging.PixelFormat FORMAT = System.Drawing.Imaging.PixelFormat.Format32bppArgb;

        public event CpEventDelegate CpEvent;

        Random rnd;

        int pictureBoxStoreImage_Left = 0;
        int pictureBoxStoreImage_Top = 0;

        public ColorPalette()
        {
            InitializeComponent();

            pictureBox1.Image = new Bitmap(pictureBox1.Width - 10, pictureBox1.Height, FORMAT);
            pictureBox2.Image = new Bitmap(pictureBox2.Width - 10, pictureBox2.Height, FORMAT);
            pictureBoxArgb.Image = new Bitmap(pictureBoxArgb.Width, pictureBoxArgb.Height, FORMAT);
            pictureBoxStore.Image = new Bitmap(pictureBoxStore.Width, pictureBoxStore.Height, FORMAT);

            rnd = new Random(Environment.TickCount);
            initPictureBoxArgb();
            initPictureBoxStoreArgb();

        }

        private void initPictureBoxStoreArgb()
        {
            Bitmap bmp = (Bitmap)pictureBoxStore.Image;
            int a = 0;
            int r = 254;
            int g = 254;
            int b = 254;
            for (int x = 0; x < bmp.Width; x++)
            {
                for(int y=0; y< bmp.Height;y++)
                {
                    bmp.SetPixel(x,y,Color.FromArgb(a,r,g,b));
                }
        }
        }

        private void initPictureBoxArgb()
        {
            Bitmap bmp = (Bitmap)pictureBoxArgb.Image;
            int a = 255;
            int r = 5;
            int g = 5;
            int b = 5;

            int step = 15;

            /*
            Graphics grf = Graphics.FromImage(bmp);

            for (int x = 0; x < bmp.Width - step; x +=step)
            {
                for (int y = 0; y < bmp.Height - step; y +=step)
                {
                    a = rnd.Next(150, 250);
                    r = rnd.Next(5, 250);
                    g = rnd.Next(5, 250);
                    b = rnd.Next(5, 250);
                    Brush brush = new SolidBrush(Color.FromArgb(a,r,g,b));
                    grf.FillRectangle(brush,x,y,step,step);
                    brush.Dispose();
                }
            }
            grf.Dispose();
             * */


            for (int x = 0; x < bmp.Width - step; x += step)
            {
                for (int y = 0; y < bmp.Height - step; y += step)
                {
                    if (checkBoxAFix.Checked)
                    {
                        a = trackBarA.Value;
                    }
                    else
                    {
                        a = rnd.Next(150, 250);
                    }
                    r = rnd.Next(5, 250);
                    g = rnd.Next(5, 250);
                    b = rnd.Next(5, 250);
                    for (int xx = x; xx < x + step; xx++)
                    {
                        for (int yy = y; yy < y + step; yy++)
                        {
                            bmp.SetPixel(xx,yy,Color.FromArgb(a,r,g,b));
                        }
                    }
                }
            }

        }

        private void trackBarA_Scroll(object sender, EventArgs e)
        {
            trackBar__Scrolled();
        }

        private void trackBarR_Scroll(object sender, EventArgs e)
        {
            trackBar__Scrolled();
        }

        private void trackBarG_Scroll(object sender, EventArgs e)
        {
            trackBar__Scrolled();
        }

        private void trackBarB_Scroll(object sender, EventArgs e)
        {
            trackBar__Scrolled();
        }

        private void trackBar__Scrolled()
        {
            int a = trackBarA.Value;
            int r = trackBarR.Value;
            int g = trackBarG.Value;
            int b = trackBarB.Value;

            string s = String.Format("({0},{1},{2},{3})", a,r,g,b);
                
            labelARGB.Text = s;

            PictureBox[] pbs = { pictureBox1, pictureBox2 };

            foreach (PictureBox pb in pbs)
            {
                Bitmap bmp = (Bitmap)pb.Image;
                for (int x = 0; x < bmp.Width; x++)
                {
                    for (int y = 0; y < bmp.Height; y++)
                    {
                        bmp.SetPixel(x,y,Color.FromArgb(a,r,g,b));
                    }
                }
                pb.Refresh();
            }
        }

        public Color getCpColor()
        {
            return Color.FromArgb(trackBarA.Value, trackBarR.Value, trackBarG.Value, trackBarB.Value);
        }

        private void RaiseCpEvent() { /* ... */ }

        private void buttonSet_Click(object sender, EventArgs e)
        {
            // Safely invoke an event:
            CpEventDelegate temp = CpEvent;

            if (temp != null)
            {
                temp(this, e);
            }

            Bitmap bmp = (Bitmap)pictureBoxStore.Image;
            pictureBoxStoreUpdate(bmp);

        }

        void pictureBoxStoreUpdate(Bitmap bmp)
        {
            if (checkBoxStoreStore.Checked == false)
            {
                return;
            }

            Color c = getCpColor();
            if (existColorInBitmap(c,bmp))
            {
                return;
            }

            int step = 10;
            if (pictureBoxStoreImage_Left > bmp.Width - 1)
            {
                pictureBoxStoreImage_Left = 0;
                pictureBoxStoreImage_Top += step;
            }
            if (pictureBoxStoreImage_Left + step > bmp.Width)
            {
                pictureBoxStoreImage_Left = 0;
                pictureBoxStoreImage_Top += step;
            }
            if (pictureBoxStoreImage_Top > bmp.Height - 1)
            {
                pictureBoxStoreImage_Left = 0;
                pictureBoxStoreImage_Top = 0;
            }
            if (pictureBoxStoreImage_Top + step > bmp.Height)
            {
                pictureBoxStoreImage_Left = 0;
                pictureBoxStoreImage_Top = 0;
            }

            for (int x = pictureBoxStoreImage_Left; x < pictureBoxStoreImage_Left + step; x++)
            {
                for (int y = pictureBoxStoreImage_Top; y < pictureBoxStoreImage_Top + step; y++)
                {
                    bmp.SetPixel(x, y, getCpColor());
                }
            }

            pictureBoxStore.Refresh();
            pictureBoxStoreImage_Left += step;
        }

        bool existColorInBitmap(Color color, Bitmap bmp)
        {
            bool result = false;
            for (int x = 0; x < bmp.Width; x++)
            {
                for (int y = 0; y < bmp.Height; y++)
                {
                    if (sameArgbOfColor(color, bmp.GetPixel(x,y)))
                    {
                        result = true;
                        break;
                    }
                }
            }
            return result;
        }

        bool sameArgbOfColor(Color color1, Color color2)
        {
            bool result = false;
            if (color1.A == color2.A &&
                color1.R == color2.R &&
                color1.G == color2.G &&
                color1.B == color2.B)
            {
                result = true;
            }

            return result;
        }

        private void pictureBoxArgb_MouseDown(object sender, MouseEventArgs e)
        {
            Bitmap bmp = (Bitmap)pictureBoxArgb.Image;
            int a = bmp.GetPixel(e.X, e.Y).A;
            int r = bmp.GetPixel(e.X, e.Y).R;
            int g = bmp.GetPixel(e.X, e.Y).G;
            int b = bmp.GetPixel(e.X, e.Y).B;

            setTrackBars(a, r, g, b);
        }

        private void setTrackBars(int a, int r, int g, int b)
        {
            trackBarA.Value = a;
            trackBarR.Value = r;
            trackBarG.Value = g;
            trackBarB.Value = b;

            trackBar__Scrolled();
        }

        private void pictureBoxStore_MouseDown(object sender, MouseEventArgs e)
        {
            Bitmap bmp = (Bitmap)pictureBoxStore.Image;
            int a = bmp.GetPixel(e.X, e.Y).A;
            int r = bmp.GetPixel(e.X, e.Y).R;
            int g = bmp.GetPixel(e.X, e.Y).G;
            int b = bmp.GetPixel(e.X, e.Y).B;

            setTrackBars(a, r, g, b);


            if (checkBoxSet.Checked == true)
            {
                // Safely invoke an event:
                CpEventDelegate temp = CpEvent;

                if (temp != null)
                {
                    temp(this, e);
                }
            }

        }

        private void pictureBox1_Click(object sender, EventArgs e)
        {
            initPictureBoxArgb();
            pictureBoxArgb.Refresh();
        }

        private void pictureBox2_Click(object sender, EventArgs e)
        {
            flipTrackBarRgb();
        }

        private void flipTrackBarRgb()
        {
            int a = trackBarA.Value;
            int r = 255 - trackBarR.Value;
            int g = 255 - trackBarG.Value;
            int b = 255 - trackBarB.Value;

            setTrackBars(a, r, g, b);
        }

        private void buttonStoreSave_Click(object sender, EventArgs e)
        {
            SaveFileDialog sfd = new SaveFileDialog();

            sfd.Filter = "ファイル(*.png)|*.png";
            sfd.Title = "保存先のファイルを選択してください";
            sfd.FileName = "paletteStore.png";

            if (sfd.ShowDialog() == DialogResult.OK)
            {
                Bitmap bmp = (Bitmap)pictureBoxStore.Image;
                bmp.Save(sfd.FileName, System.Drawing.Imaging.ImageFormat.MemoryBmp);
                // bmp.Dispose();
            }
        }

        private void buttonStoreLoad_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "画像ファイル(*.bmp;*.jpg;*.jpeg;*.png)|*.bmp;*.jpg;*.jpeg;*.png";
            ofd.Title = "開くファイルを選択してください";
            ofd.FileName = "paletteStore";
            ofd.RestoreDirectory = true;

            if (ofd.ShowDialog() == DialogResult.OK)
            {
                System.IO.FileStream stream = new System.IO.FileStream(ofd.FileName, System.IO.FileMode.Open);
                Bitmap bmp = new Bitmap(stream);
                stream.Close();
                //pictureBoxStore.Image = bmp;
                copyBitmap(bmp, (Bitmap)pictureBoxStore.Image);
                bmp.Dispose();
                pictureBoxStore.Refresh();
            }
        }

        private void buttonPaletteSave_Click(object sender, EventArgs e)
        {
            SaveFileDialog sfd = new SaveFileDialog();

            sfd.Filter = "ファイル(*.png)|*.png";
            sfd.Title = "保存先のファイルを選択してください";
            sfd.FileName = "palette.png";

            if (sfd.ShowDialog() == DialogResult.OK)
            {
                Bitmap bmp = (Bitmap)pictureBoxArgb.Image;
                bmp.Save(sfd.FileName, System.Drawing.Imaging.ImageFormat.MemoryBmp);
                // bmp.Dispose();
            }
        }

        private void buttonPaletteLoad_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "画像ファイル(*.bmp;*.jpg;*.jpeg;*.png)|*.bmp;*.jpg;*.jpeg;*.png";
            ofd.Title = "開くファイルを選択してください";
            ofd.FileName = "palette";
            ofd.RestoreDirectory = true;

            if (ofd.ShowDialog() == DialogResult.OK)
            {
                System.IO.FileStream stream = new System.IO.FileStream(ofd.FileName, System.IO.FileMode.Open);
                Bitmap bmp = new Bitmap(stream);
                stream.Close();
                //pictureBoxArgb.Image = bmp;
                copyBitmap(bmp, (Bitmap)pictureBoxArgb.Image);
                bmp.Dispose();
                pictureBoxArgb.Refresh();
            }
        }

        private void copyBitmap(Bitmap from, Bitmap to)
        {
            int minx = Math.Min(from.Width, to.Width);
            int miny = Math.Min(from.Height, to.Height);
            for (int x = 0; x < minx; x++)
            {
                for (int y = 0; y < miny; y++)
                {
                    to.SetPixel(x, y, from.GetPixel(x, y));
                }
            }

        }

        private void buttonRYGCBM_Click(object sender, EventArgs e)
        {
            Color[] colors = {Color.Red,Color.Yellow,Color.Green,Color.Cyan,Color.Blue,Color.Magenta,
                                 Color.FromArgb(255,250,250,250),
                                 Color.FromArgb(255,127,127,127),
                                 Color.FromArgb(255,5,5,5)};

            object dummy = new object();
            EventArgs dummyea = new EventArgs();
            foreach (Color color in colors)
            {
                setTrackBars(color.A, color.R, color.G, color.B);
                buttonSet_Click(dummy, dummyea);
            }
        }

    }
}
