﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Reflection;


namespace MyNote001
{
    public delegate void WatchMyMacrosEventDelegate(string str);
    public delegate void ConvertChangedEventDelegate(string sa, string sb, string sc, string sd, string se, string sf, string sg, string sh, string sk, string sl, string sm, string sn);
    public delegate void B_MakeApngChangedEventDelegate(bool b);
    public delegate void I_ApngstepChangedEventDelegate(int i);
    public delegate void MacrosCommandEnabledEventDelegate(bool b);

    public class MyMacros
    {
        public event WatchMyMacrosEventDelegate WatchMyMacrosEvent;
        public event ConvertChangedEventDelegate ConvertChangedEvent;
        public event B_MakeApngChangedEventDelegate B_MakeApngChangedEvent;
        public event I_ApngstepChangedEventDelegate I_ApngstepChangedEvent;
        public event MacrosCommandEnabledEventDelegate MacrosCommandEnabledEvent;

        private bool b_makeApng;
        const int CommandEnded = -1;
        List<byte[]> pngList;
        private string pngsFolder;
        private int counter;
        private int i_apngstep = 10;

        const string OutputFolder = "output";

        Form1 form;

        RichTextBox richTextBoxNotExecuted = new RichTextBox();

        public const string Comment = "//";
        public const string Size = "size";
        public const string Color = "color";
        public const string PenSize = "pensize";
        public const string PenMode = "penmode";
        public const string PenKind = "penkind";
        public const string DotDrawMode = "ddm";
        public const string DDMSimpleMixRate = "ddmsmr";
        public const string TargetLayer = "targetlayer";

        public const string MouseDown = "mousedown";
        public const string MouseMove = "mousemove";
        public const string MouseUp = "mouseup";

        public const string AntiAliasing = "antialiasing";
        public const string AntiAliasingType = "aat";

        const string ConvertOn = "converton";
        const string ConvertOff = "convertoff";
        const string Convert = "convert";
        const string ConvertRotate = "convertrotate";
        const string Cr = "cr";

        const string SetConvertABCD = "setconvertabcd";
        const string SCA = "sca";
        const string SetConvertABCDRotate = "setconvertabcdrotate";
        const string SCAR = "scar";
        const string SetConvertEF = "setconvertef";
        const string SCE = "sce";
        const string SetConvertGH = "setconvertgh";
        const string SCG = "scg";

        const string ConvertReset = "convertreset";
        const string SetConvertKL = "setconvertkl";
        const string SCK = "sck";
        const string SetConvertMN = "setconvertmn";
        const string SCM = "scm";

        const string Alert = "alert";

        public const string DotDrawCycleReset = "dotdrawcyclereset";
        public const string DotDrawCycle = "dotdrawcycle";

        const string WriteLaster = "writelaster";
        const string WriteLaster2 = "writelaster2";

        const string ApngStep = "apngstep";

        bool bool_convert = false;

        double d_a;
        double d_b;
        double d_c;
        double d_d;
        double d_e;
        double d_f;
        double d_g;
        double d_h;
        double d_k;
        double d_l;
        double d_m;
        double d_n;

        private static MyMacros _singleton = null;

        private bool b_cancel;
        private FormProgress fp;

        public static MyMacros Singleton(Form1 form1)
        {
            if(_singleton == null)
            {
                _singleton = new MyMacros(form1);
            }
            return _singleton;
        }

        private MyMacros(Form1 form1)
        {
            form = form1;
            convertReset();
            convertoff();
        }

        ///////////////////////

        private void setI_Apngstep(int step)
        {
            this.i_apngstep = step;
            invokeI_ApngstepChangedEvent(step);
        }

        public int getI_Apngstep()
        {
            return this.i_apngstep;
        }

        private void invokeI_ApngstepChangedEvent(int step)
        {
            I_ApngstepChangedEventDelegate temp = I_ApngstepChangedEvent;

            if (temp != null)
            {
                temp(step);
            }
        }

        ///////////////////////

        public void setB_MakeApng(bool b)
        {
            this.b_makeApng = b;
            invokeB_MakeApngChangedEvent(b);
        }

        public bool getB_MakeApng()
        {
            return this.b_makeApng;
        }


        private void invokeB_MakeApngChangedEvent(bool b)
        {
            B_MakeApngChangedEventDelegate temp = B_MakeApngChangedEvent;

            if (temp != null)
            {
                temp(b);
            }
        }

        ///////////////////////

        public void invokeWatchMyMacrosEvent(string str)
        {
            // Safely invoke an event:
            WatchMyMacrosEventDelegate temp = WatchMyMacrosEvent;

            if (temp != null)
            {
                temp(str);
            }
        }

        public void invokeConvertChangedEvent()
        {
            // Safely invoke an event:
            ConvertChangedEventDelegate temp = ConvertChangedEvent;

            if (temp != null)
            {
                string sa, sb, sc, sd, se, sf, sg, sh;
                string sk, sl, sm, sn;
                if (bool_convert)
                {
                    string format = "0.000000";
                    sa = d_a.ToString(format);
                    sb = d_b.ToString(format);
                    sc = d_c.ToString(format);
                    sd = d_d.ToString(format);
                    se = d_e.ToString(format);
                    sf = d_f.ToString(format);
                    sg = d_g.ToString(format);
                    sh = d_h.ToString(format);
                    sk = d_k.ToString(format);
                    sl = d_l.ToString(format);
                    sm = d_m.ToString(format);
                    sn = d_n.ToString(format);
                }
                else
                {
                    sa = "a";
                    sb = "b";
                    sc = "c";
                    sd = "d";
                    se = "e";
                    sf = "f";
                    sg = "g";
                    sh = "h";
                    sk = "k";
                    sl = "l";
                    sm = "m";
                    sn = "n";
                }
                temp(sa, sb, sc, sd, se, sf, sg, sh, sk, sl, sm, sn);
            }
        }

        public string nowStatus()
        {
            string str = "//, now status";
            str += Environment.NewLine;
            str += getSize();
            str += Environment.NewLine;
            str += getColor();
            str += Environment.NewLine;
            str += getPensize();
            str += Environment.NewLine;

            str += getPenMode();
            str += Environment.NewLine;
            str += getPenKind();
            str += Environment.NewLine;

            str += getDotDrawMode();
            str += Environment.NewLine;
            str += getDotDrawModeSimpleMixRate();
            str += Environment.NewLine;

            str += getTargetLayer();
            str += Environment.NewLine;

            str += getAntiAliasing();
            str += Environment.NewLine;
            str += getAntiAliasingType();
            str += Environment.NewLine;

            return str;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="commands"></param>
        public void evaluate(string[] commands)
        {
            richTextBoxNotExecuted.Text = "";

            if (b_makeApng)
            {
                try
                {
                    pngList = new List<byte[]>();
                    counter = 1;
                    makePngsFolder();
                }
                catch (Exception e)
                {
                    MessageBox.Show(e.ToString());
                    return;
                }
            }

            fp = new FormProgress();
            fp.COUNTMAX = commands.Length;
            fp.Show();
            b_cancel = false;
            invokeMacrosCommandEnabledEvent(false);
            form.set_macrosCommandToolStripMenuItem_Enabled(false);

            int split = 5000;
            int n = commands.Length / split;
            for (int i = 0; i < n + 1; i++)
            {
                string[] substring;
                if (i < n)
                {
                    substring = new string[split];
                    Array.Copy(commands, i * split,substring, 0, split);
                    evaluatesub(substring, i * split);
                    if (b_cancel)
                    {
                        break;
                    }
                    //MessageBox.Show(string.Format("{0}/{1} finished", (i + 1) * split, commands.Length));
                }
                else
                {
                    substring = new string[commands.Length % split];
                    Array.Copy(commands, i * split, substring, 0, commands.Length % split);
                    evaluatesub(substring, i * split);
                    if (b_cancel)
                    {
                        break;
                    }
                    //MessageBox.Show(string.Format("{0}/{1} finished", i * split + commands.Length % split, commands.Length));
                }
            }

            if (b_makeApng)
            {
                makeApng(MyMacros.CommandEnded);
            }

            fp.Close();
            invokeMacrosCommandEnabledEvent(true);
            form.set_macrosCommandToolStripMenuItem_Enabled(true);
            
            // とりあえず。
            string[] temps = {"dummy", "-1000", "-1000"};
            mouseup(temps, -1);
            

            if (richTextBoxNotExecuted.Text != "")
            {
                MessageBox.Show(richTextBoxNotExecuted.Text, "not executed");
            }

        }

        private void evaluatesub(string[] commands, int index)
        {
            char[] separator = { ',' };
            for (int i = 0; i < commands.Length; i++)
            {
                Application.DoEvents();
                if (fp.CANCEL)
                {
                    b_cancel = true;
                    break;
                }
                fp.COUNT = index + i + 1;
                fp.INFORMATION = commands[i];
                
                if (b_makeApng)
                {
                    makeApng(index + i + 1);
                }

                if (commands[i].Length < 1)
                    continue;
                string[] strs = commands[i].Split(separator);
                try
                {
                    evaluateCommand(strs, index + i + 1);
                }
                catch (Exception e)
                {
                    richTextBoxNotExecuted.Text += string.Format("line {0}: {1}", i + 1, commands[i]);
                    richTextBoxNotExecuted.Text += Environment.NewLine;
                }
            }
        }

        private void evaluateCommand(string[] strs, int line)
        {
            if (strs.Length < 1)
                throw new Exception();

            string command = string.Format("{0}", strs[0]);
            command = command.Trim();
            command = command.ToLower();
            switch (command)
            {
                case MyMacros.Comment:
                    break;
                case MyMacros.Size:
                    setSize(strs, line);
                    break;
                case MyMacros.Color:
                    setColor(strs, line);
                    break;
                case MyMacros.PenSize:
                    setPenSize(strs, line);
                    break;
                case MyMacros.PenMode:
                    setPenMode(strs, line);
                    break;
                case MyMacros.PenKind:
                    setPenKind(strs, line);
                    break;
                case MyMacros.DotDrawMode:
                    setDotDrawMode(strs, line);
                    break;
                case MyMacros.DDMSimpleMixRate:
                    setDotDrawModeSimpleMixRate(strs, line);
                    break;
                case MyMacros.TargetLayer:
                    setTargetLayer(strs, line);
                    break;

                case MyMacros.MouseDown:
                    mousedown(strs, line);
                    break;
                case MyMacros.MouseMove:
                    mousemove(strs, line);
                    break;
                case MyMacros.MouseUp:
                    mouseup(strs, line);
                    break;

                case MyMacros.AntiAliasing:
                    setAntiAliasing(strs, line);
                    break;
                case MyMacros.AntiAliasingType:
                    setAntiAliasingType(strs, line);
                    break;

                case MyMacros.ConvertOn:
                    converton();
                    break;
                case MyMacros.ConvertOff:
                    convertoff();
                    break;
                case MyMacros.Convert:
                    setConvert(strs, line);
                    break;
                case MyMacros.ConvertRotate:
                case MyMacros.Cr:
                    setConvertRotate(strs, line);
                    break;

                case MyMacros.SetConvertABCD:
                case MyMacros.SCA:
                    setConvertAbcd(strs, line);
                    break;
                case MyMacros.SetConvertABCDRotate:
                case MyMacros.SCAR:
                    setConvertAbcdRotate(strs, line);
                    break;
                case MyMacros.SetConvertEF:
                case MyMacros.SCE:
                    setConvertEf(strs, line);
                    break;
                case MyMacros.SetConvertGH:
                case MyMacros.SCG:
                    setConvertGh(strs, line);
                    break;

                case MyMacros.ConvertReset:
                    convertReset();
                    break;
                case MyMacros.SetConvertKL:
                case MyMacros.SCK:
                    setConvertKl(strs, line);
                    break;
                case MyMacros.SetConvertMN:
                case MyMacros.SCM:
                    setConvertMn(strs, line);
                    break;

                case MyMacros.Alert:
                    alert(line);
                    break;
                case MacrosWatchAsCommand.USE_STRINGBUILDER:
                    invokeWatchMyMacrosEvent(MacrosWatchAsCommand.USE_STRINGBUILDER);
                    break;

                case MyMacros.DotDrawCycleReset:
                    dotDrawCycleReset();
                    break;
                case MyMacros.DotDrawCycle:
                    setDotDrawCycle(strs, line);
                    break;

                case MyMacros.WriteLaster:
                    writeLaster(strs, line);
                    break;
                case MyMacros.WriteLaster2:
                    writeLaster2(strs, line);
                    break;
                case MyMacros.ApngStep:
                    setApngstep(strs, line);
                    break;
                default:
                    throw new Exception();
                    break;
            }

        }

        private void invokeMacrosCommandEnabledEvent(bool b)
        {
            MacrosCommandEnabledEventDelegate temp = MacrosCommandEnabledEvent;

            if (temp != null)
            {
                temp(b);
            }
        }

        ////////////////////////////////

        private void exceptionCatched(Exception e, int line)
        {
            string s = string.Format("line {0}: ", line) + Environment.NewLine + e.ToString();
            MessageBox.Show(s);
            throw e;
        }

        ////////////////////////////////

        private string getSize()
        {
            string str = string.Format("{0},{1},{2}",
                MyMacros.Size,
                form.macrosGetSize().Width,
                form.macrosGetSize().Height);
            return str;
        }

        private void setSize(string[] strs, int line)
        {
            if (strs.Length < 3)
                throw new Exception();

            if (b_makeApng)
                throw new Exception("size command failed because makeApng is acting");

            try
            {
                int width = int.Parse(strs[1]);
                int height = int.Parse(strs[2]);
                form.macrosSetSize(width, height);
            }
            catch (Exception e)
            {
                exceptionCatched(e, line);
            }
        }

        //////////

        private string getColor()
        {
            string str = string.Format("{0},{1},{2},{3},{4}",
                MyMacros.Color,
                form.macrosGetColor()[0],
                form.macrosGetColor()[1],
                form.macrosGetColor()[2],
                form.macrosGetColor()[3]);

            return str;
        }

        private void setColor(string[] strs, int line)
        {
            if (strs.Length < 5)
                throw new Exception();

            try
            {
                int a = int.Parse(strs[1]);
                int r = int.Parse(strs[2]);
                int g = int.Parse(strs[3]);
                int b = int.Parse(strs[4]);
                form.macrosSetColor(a, r, g, b);
            }
            catch (Exception e)
            {
                exceptionCatched(e, line);
            }
        }

        //////////

        private string getPensize()
        {
            string str = string.Format("{0},{1}",
                MyMacros.PenSize,
                form.macrosGetPenSize());
            return str;
        }

        private void setPenSize(string[] strs, int line)
        {
            if (strs.Length < 2)
                throw new Exception();

            try
            {
                int newsize = int.Parse(strs[1]);
                form.macrosSetPenSize(newsize);
            }
            catch (Exception e)
            {
                exceptionCatched(e, line);
            }
        }

        //////////

        private string getPenMode()
        {
            string str = string.Format("{0},{1}",
                MyMacros.PenMode,
                form.macrosGetPenMode());
            return str;
        }

        private void setPenMode(string[] strs, int line)
        {
            if (strs.Length < 2)
                throw new Exception();

            try
            {
                form.macrosSetPenMode(strs[1]);
            }
            catch (Exception e)
            {
                exceptionCatched(e, line);
            }
        }

        //////////

        private string getPenKind()
        {
            string str = string.Format("{0},{1}",
                MyMacros.PenKind,
                form.macrosGetPenKind());
            return str;
        }

        private void setPenKind(string[] strs, int line)
        {
            if (strs.Length < 2)
                throw new Exception();

            try
            {
                form.macrosSetPenKind(strs[1]);
            }
            catch (Exception e)
            {
                exceptionCatched(e, line);
            }
        }

        //////////

        private string getDotDrawMode()
        {
            string str = string.Format("{0},{1}",
                MyMacros.DotDrawMode,
                form.macrosGetDotDrawMode());
            return str;
        }

        private void setDotDrawMode(string[] strs, int line)
        {
            if (strs.Length < 2)
                throw new Exception();

            try
            {
                form.macrosSetDotDrawMode(strs[1]);
            }
            catch (Exception e)
            {
                exceptionCatched(e, line);
            }
        }

        //////////

        private string getDotDrawModeSimpleMixRate()
        {
            string str = string.Format("{0},{1}",
                MyMacros.DDMSimpleMixRate,
                form.macrosGetDotDrawModeSimpleMixRate());
            return str;
        }

        private void setDotDrawModeSimpleMixRate(string[] strs, int line)
        {
            if (strs.Length < 2)
                throw new Exception();

            try
            {
                form.macrosSetDotDrawModeSimpleMixRate(strs[1]);
            }
            catch (Exception e)
            {
                exceptionCatched(e, line);
            }
        }

        //////////

        private string getTargetLayer()
        {
            string str = string.Format("{0},{1}",
                MyMacros.TargetLayer,
                form.macrosGetTargetLayer());
            return str;
        }

        private void setTargetLayer(string[] strs, int line)
        {
            if (strs.Length < 2)
                throw new Exception();

            try
            {
                form.macrosSetTargetLayer(strs[1]);
            }
            catch (Exception e)
            {
                exceptionCatched(e, line);
            }
        }

        //////////

        private void mousedown(string[] strs, int line)
        {
            if (strs.Length < 3)
                throw new Exception();
            try
            {
                int x = int.Parse(strs[1]);
                int y = int.Parse(strs[2]);
                if (bool_convert)
                {
                    int[] xy = convertXY(x, y);
                    x = xy[0];
                    y = xy[1];
                }
                form.macrosMouseDown(x, y);
            }
            catch (Exception e)
            {
                exceptionCatched(e, line);
            }
        }

        private void mousemove(string[] strs, int line)
        {
            if (strs.Length < 3)
                throw new Exception();
            try
            {
                int x = int.Parse(strs[1]);
                int y = int.Parse(strs[2]);
                if (bool_convert)
                {
                    int[] xy = convertXY(x, y);
                    x = xy[0];
                    y = xy[1];
                }
                form.macrosMouseMove(x, y);
            }
            catch (Exception e)
            {
                exceptionCatched(e, line);
            }
        }

        private void mouseup(string[] strs, int line)
        {
            if (strs.Length < 3)
                throw new Exception();
            try
            {
                int x = int.Parse(strs[1]);
                int y = int.Parse(strs[2]);
                if (bool_convert)
                {
                    int[] xy = convertXY(x, y);
                    x = xy[0];
                    y = xy[1];
                }
                form.macrosMouseUp(x, y);
            }
            catch (Exception e)
            {
                exceptionCatched(e, line);
            }
        }

        //////////

        private string getAntiAliasing()
        {
            string str = string.Format("{0},{1}",
                MyMacros.AntiAliasing,
                form.macrosGetAntiAliasing());

            return str;
        }

        private void setAntiAliasing(string[] strs, int line)
        {
            if (strs.Length < 2)
                throw new Exception();

            try
            {
                form.macrosSetAntiAliasing(strs[1]);
            }
            catch (Exception e)
            {
                exceptionCatched(e, line);
            }
        }

        //////////

        private string getAntiAliasingType()
        {
            string str = string.Format("{0},{1}",
                MyMacros.AntiAliasingType,
                form.macrosGetAntiAliasingType());
            return str;
        }

        private void setAntiAliasingType(string[] strs, int line)
        {
            if (strs.Length < 2)
                throw new Exception();

            try
            {
                form.macrosSetAntiAliasingType(strs[1]);
            }
            catch (Exception e)
            {
                exceptionCatched(e, line);
            }
        }

        ////////// //////////

        private void converton()
        {
            bool_convert = true;
            invokeConvertChangedEvent();
        }

        private void convertoff()
        {
            bool_convert = false;
            invokeConvertChangedEvent();
        }

        private void setConvert(string[] strs, int line)
        {
            if (strs.Length < 9)
            {
                convertoff();
                throw new Exception();
            }

            try
            {
                d_a = double.Parse(strs[1]);
                d_b = double.Parse(strs[2]);
                d_c = double.Parse(strs[3]);
                d_d = double.Parse(strs[4]);
                d_e = double.Parse(strs[5]);
                d_f = double.Parse(strs[6]);
                d_g = double.Parse(strs[7]);
                d_h = double.Parse(strs[8]);
                converton();
            }
            catch (Exception e)
            {
                convertoff();
                exceptionCatched(e, line);
            }
        }

        private int[] convertXY(int x, int y)
        {
            double dx = d_k * (d_a * (x - d_e) + d_b * (y - d_f)) + d_m * d_g;
            double dy = d_l * (d_c * (x - d_e) + d_d * (y - d_f)) + d_n * d_h;
            int[] xy = { (int)dx, (int)dy };
            return xy;
        }

        private void setConvertRotate(string[] strs, int line)
        {
            if (strs.Length < 4)
            {
                convertoff();
                throw new Exception();
            }

            try
            {
                double r = 2 * Math.PI * (double.Parse(strs[1]) / 360.0);
                d_a = Math.Cos(r);
                d_b = 0 - Math.Sin(r);
                d_c = Math.Sin(r);
                d_d = Math.Cos(r);
                d_e = double.Parse(strs[2]);
                d_f = double.Parse(strs[3]);
                d_g = double.Parse(strs[2]);
                d_h = double.Parse(strs[3]);
                converton();
            }
            catch (Exception e)
            {
                convertoff();
                exceptionCatched(e, line);
            }
        }

        //////////

        private void setConvertAbcd(string[] strs, int line)
        {
            if (strs.Length < 5)
            {
                convertoff();
                throw new Exception();
            }

            try
            {
                d_a = double.Parse(strs[1]);
                d_b = double.Parse(strs[2]);
                d_c = double.Parse(strs[3]);
                d_d = double.Parse(strs[4]);
                converton();
            }
            catch (Exception e)
            {
                convertoff();
                exceptionCatched(e, line);
            }
        }

        private void setConvertAbcdRotate(string[] strs, int line)
        {
            if (strs.Length < 2)
            {
                convertoff();
                throw new Exception();
            }

            try
            {
                double r = 2 * Math.PI * (double.Parse(strs[1]) / 360.0);
                d_a = Math.Cos(r);
                d_b = 0 - Math.Sin(r);
                d_c = Math.Sin(r);
                d_d = Math.Cos(r);
                converton();
            }
            catch (Exception e)
            {
                convertoff();
                exceptionCatched(e, line);
            }
        }

        private void setConvertEf(string[] strs, int line)
        {
            if (strs.Length < 3)
            {
                convertoff();
                throw new Exception();
            }

            try
            {
                d_e = double.Parse(strs[1]);
                d_f = double.Parse(strs[2]);
                converton();
            }
            catch (Exception e)
            {
                convertoff();
                exceptionCatched(e, line);
            }
        }

        private void setConvertGh(string[] strs, int line)
        {
            if (strs.Length < 3)
            {
                convertoff();
                throw new Exception();
            }

            try
            {
                d_g = double.Parse(strs[1]);
                d_h = double.Parse(strs[2]);
                converton();
            }
            catch (Exception e)
            {
                convertoff();
                exceptionCatched(e, line);
            }
        }

        private void convertReset()
        {
            d_a = 1.0;
            d_b = 0.0;
            d_c = 0.0;
            d_d = 1.0;
            d_e = 0.0;
            d_f = 0.0;
            d_g = 0.0;
            d_h = 0.0;
            d_k = 1.0;
            d_l = 1.0;
            d_m = 1.0;
            d_n = 1.0;
            converton();
        }

        private void setConvertKl(string[] strs, int line)
        {
            if (strs.Length < 3)
            {
                convertoff();
                throw new Exception();
            }

            try
            {
                d_k = double.Parse(strs[1]);
                d_l = double.Parse(strs[2]);
                converton();
            }
            catch (Exception e)
            {
                convertoff();
                exceptionCatched(e, line);
            }
        }

        private void setConvertMn(string[] strs, int line)
        {
            if (strs.Length < 3)
            {
                convertoff();
                throw new Exception();
            }

            try
            {
                d_m = double.Parse(strs[1]);
                d_n = double.Parse(strs[2]);
                converton();
            }
            catch (Exception e)
            {
                convertoff();
                exceptionCatched(e, line);
            }
        }

        private void alert(int line)
        {
            MessageBox.Show(string.Format("line {0}", line), MyMacros.Alert);
        }

        private void dotDrawCycleReset()
        {
            try
            {
                DotDrawCycle ddc1 = new DotDrawCycle();
                DotDrawCycle ddc2 = new DotDrawCycle();
                DotDrawCycle ddc3 = new DotDrawCycle();
                DotDrawCycle ddc4 = new DotDrawCycle();
                form.macrosSetDotDrawCycle(ddc1, ddc2, ddc3, ddc4);
            }
            catch (Exception e)
            {
                throw e;
            }
        }

        private void setDotDrawCycle(string[] strs, int line)
        {         
            if (strs.Length < 2)
                throw new Exception();

            try
            {
                int n = int.Parse(strs[1]);
                if (n < 1)
                    throw new Exception("too small");
                if (n > 64)
                    throw new Exception("too large");

                bool[,] bools = new bool[n, n];
                for (int i = 0; i < n; i++)
                {
                    for (int j = 0; j < n; j++)
                    {
                        bools[i, j] = true;
                    }
                }

                DotDrawCycle ddc1 = new DotDrawCycle();
                DotDrawCycle ddc2 = new DotDrawCycle();
                DotDrawCycle ddc3 = new DotDrawCycle();
                DotDrawCycle ddc4 = new DotDrawCycle();

                ddc1.SetDotDrawCycle(bools, n, n);
                ddc2.SetDotDrawCycle(bools, n, n);
                ddc3.SetDotDrawCycle(bools, n, n);
                ddc4.SetDotDrawCycle(bools, n, n);
                form.macrosSetDotDrawCycle(ddc1, ddc2, ddc3, ddc4);

            }
            catch (Exception e)
            {
                exceptionCatched(e, line);
            }
        }

        //////////

        private void makeApng(int i)
        {
            int step = i_apngstep;

            if (step == 1 || (i > 0 && i % step == 1) || i == MyMacros.CommandEnded)
            {
                Bitmap bmp = form.getLasterBitmap();
                byte[] result = null;
                using (MemoryStream stream = new MemoryStream())
                {
                    bmp.Save(stream, ImageFormat.Png);
                    result = stream.ToArray();
                }

                //makeApngSubが未実装なのでコメントアウト。
                //pngList.Add(result);

                string fileName = string.Format("{0:00000}.png", counter);
                string filePath = pngsFolder + Path.DirectorySeparatorChar.ToString() + fileName;
                savePngByte(filePath, result);
                counter++;

                if (i == MyMacros.CommandEnded)
                {
                    // File.WriteAllBytes("test.png", result);
                    makeApngSub(pngList);
                }
            }
        }

        private void makeApngSub(List<byte[]> pngList)
        {
        }

        private void makePngsFolder()
        {
            DateTime dt = DateTime.Now;
            string str = string.Format("{0:0000}{1:00}{2:00}{3:00}{4:00}{5:00}",
                dt.Year, dt.Month, dt.Day, dt.Hour, dt.Minute, dt.Second);

            pngsFolder =
                Path.GetDirectoryName(Assembly.GetExecutingAssembly().GetModules()[0].FullyQualifiedName) +
                Path.DirectorySeparatorChar.ToString() +
                MyMacros.OutputFolder +
                Path.DirectorySeparatorChar.ToString() +
                str;

            try
            {
                Directory.CreateDirectory(pngsFolder);
            }
            catch(Exception e)
            {
                throw e;
            }
       }

        private void savePngByte(string filePath, byte[] png)
        {
            File.WriteAllBytes(filePath, png);
        }

        //////////

        private void writeLaster(string[] strs, int line)
        {
            if (strs.Length < 5)
            {
                throw new Exception();
            }

            int startX, startY, sizeX, sizeY;

            try
            {
                startX = int.Parse(strs[1]);
                startY = int.Parse(strs[2]);
                sizeX = int.Parse(strs[3]);
                sizeY = int.Parse(strs[4]);

                sizeCheck(sizeX, sizeY);
                Bitmap from = form.getLasterBitmap();
                existCheck(from, startX, startY, sizeX, sizeY);
                Bitmap to = new Bitmap(Math.Abs(sizeX),Math.Abs(sizeY));
                writeLasterSub(from, to, startX, startY, sizeX, sizeY);
                saveBitmap(to);
                to.Dispose();
            }
            catch (Exception e)
            {
                exceptionCatched(e, line);
            }
        }

        private void writeLaster2(string[] strs, int line)
        {
            if (strs.Length < 5)
            {
                throw new Exception();
            }

            int startX, startY, sizeX, sizeY;

            try
            {
                startX = int.Parse(strs[1]);
                startY = int.Parse(strs[2]);
                sizeX = int.Parse(strs[3]);
                sizeY = int.Parse(strs[4]);

                sizeCheck(sizeX, sizeY);
                Bitmap from = form.getLasterBitmap();
                existCheck(from, startX, startY, sizeX, sizeY);
                Bitmap to = new Bitmap(Math.Abs(sizeX), Math.Abs(sizeY));
                writeLasterSub(from, to, startX, startY, sizeX, sizeY);
                Bitmap to2 = new Bitmap(to.Width * 2, to.Height * 2);
                writeLaster2Sub(to, to2);
                saveBitmap(to2);
                to.Dispose();
                to2.Dispose();
            }
            catch (Exception e)
            {
                exceptionCatched(e, line);
            }
        }

        private void sizeCheck(int sizeX, int sizeY)
        {
            if (sizeX == 0)
                throw new Exception("sizeX 0 error");
            if (sizeY == 0)
                throw new Exception("sizeY 0 error");
        }

        private void existCheck(Bitmap bmp, int startX, int startY, int sizeX, int sizeY)
        {
            if (0 <= startX && startX < bmp.Width)
            {
            }
            else
            {
                throw new Exception("error : startX");
            }

            if (0 <= startY && startY < bmp.Height)
            {
            }
            else
            {
                throw new Exception("error : startY");
            }

            int x = startX + sizeX - sizeX / Math.Abs(sizeX);
            int y = startY + sizeY - sizeY / Math.Abs(sizeY);

            if (0 <= x && x < bmp.Width)
            {
            }
            else
            {
                throw new Exception("error : startX + sizeX - sizeX / Math.Abs(sizeX)");
            }

            if (0 <= y && y < bmp.Height)
            {
            }
            else
            {
                throw new Exception("error : startY + sizeY - sizeY / Math.Abs(sizeY)");
            }
        }

        private void writeLasterSub(Bitmap from, Bitmap to, int starX, int startY, int sizeX, int sizeY)
        {
            int directionX = sizeX / Math.Abs(sizeX);
            int directionY = sizeY / Math.Abs(sizeY);

            for (int x = 0; x < to.Width; x++)
            {
                for (int y = 0; y < to.Height; y++)
                {
                    int fx = starX + x * directionX;
                    int fy = startY + y * directionY;
                    to.SetPixel(x, y, from.GetPixel(fx, fy));
                }
            }
        }

        private void writeLaster2Sub(Bitmap to, Bitmap to2)
        {
            int centerX = to2.Width / 2;
            int centerY = to2.Height / 2;

            for (int x = 0; x < to.Width; x++)
            {
                for (int y = 0; y < to.Height; y++)
                {
                    // 右下
                    to2.SetPixel(centerX + x, centerY + y, to.GetPixel(x, y));
                    // 右上
                    to2.SetPixel(centerX + x, centerY -1 - y, to.GetPixel(x, y));
                    // 左下
                    to2.SetPixel(centerX -1 - x, centerY + y, to.GetPixel(x, y));
                    // 左上
                    to2.SetPixel(centerX - 1 - x, centerY -1 - y, to.GetPixel(x, y));

                }
            }
        }

        private void saveBitmap(Bitmap bmp)
        {
            string outputfolder = getOutputFolder();

            DateTime dt = DateTime.Now;
            string filename = string.Format("{0:0000}{1:00}{2:00}{3:00}{4:00}{5:00}.png",
                dt.Year, dt.Month, dt.Day, dt.Hour, dt.Minute, dt.Second);

            string filepath = outputfolder +
                Path.DirectorySeparatorChar.ToString() +
                filename;

            while(File.Exists(filepath))
            {
               System.Threading.Thread.Sleep(300);
               dt = DateTime.Now;
               filename = string.Format("{0:0000}{1:00}{2:00}{3:00}{4:00}{5:00}.png",
                   dt.Year, dt.Month, dt.Day, dt.Hour, dt.Minute, dt.Second);
               filepath = outputfolder +
                   Path.DirectorySeparatorChar.ToString() +
                   filename;
            }

            // System.Drawing.Imaging.ImageFormat.Bmp だとアルファ値が強制255される
            bmp.Save(filepath, System.Drawing.Imaging.ImageFormat.MemoryBmp);

        }

        private string getOutputFolder()
        {
            string outputFolder =
                Path.GetDirectoryName(Assembly.GetExecutingAssembly().GetModules()[0].FullyQualifiedName) +
                Path.DirectorySeparatorChar.ToString() +
                MyMacros.OutputFolder;

            if(!Directory.Exists(outputFolder))
            try
            {
                Directory.CreateDirectory(outputFolder);
            }
            catch (Exception e)
            {
                throw e;
            }

            return outputFolder;
        }


        private void setApngstep(string[] strs, int line)
        {
            if (strs.Length < 2)
            {
                throw new Exception();
            }


            try
            {
                int step = int.Parse(strs[1]);
                if (step < 1)
                    throw new Exception("error: apngstep must be 1 or over");
                setI_Apngstep(step);
            }
            catch (Exception e)
            {
                exceptionCatched(e, line);
            }
        }
    }
}
